var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const ok$1 = (data) => ({ ok: true, data });
const err$1 = (error) => ({
  ok: false,
  error
});
const wrapThrows$1 = (fn) => (...args) => {
  try {
    return {
      ok: true,
      data: fn(...args)
    };
  } catch (error) {
    return {
      ok: false,
      error
    };
  }
};
async function makeRequest(apiHost, endpoint, method, data) {
  const url = new URL(endpoint, apiHost);
  const body = JSON.stringify(data);
  const res = wrapThrows$1(fetch)(url.toString(), {
    method,
    headers: {
      "Content-Type": "application/json"
    },
    body
  });
  if (res.ok === false)
    return err$1(res.error);
  const response = await res.data;
  const { data: innerData } = await response.json();
  if (!response.ok) {
    return err$1({
      code: "network_error",
      message: response.statusText,
      status: response.status,
      url
    });
  }
  return ok$1(innerData);
}
class ActionAPI {
  constructor(apiHost, environmentId) {
    this.apiHost = apiHost;
    this.environmentId = environmentId;
  }
  async create(actionInput) {
    return makeRequest(this.apiHost, `/api/v1/client/${this.environmentId}/actions`, "POST", actionInput);
  }
}
class DisplayAPI {
  constructor(baseUrl, environmentId) {
    this.apiHost = baseUrl;
    this.environmentId = environmentId;
  }
  async create(displayInput) {
    return makeRequest(this.apiHost, `/api/v1/client/${this.environmentId}/displays`, "POST", displayInput);
  }
  async update(displayId, displayInput) {
    return makeRequest(
      this.apiHost,
      `/api/v1/client/${this.environmentId}/displays/${displayId}`,
      "PUT",
      displayInput
    );
  }
}
class PeopleAPI {
  constructor(apiHost, environmentId) {
    this.apiHost = apiHost;
    this.environmentId = environmentId;
  }
  async create(userId) {
    return makeRequest(this.apiHost, `/api/v1/client/${this.environmentId}/people`, "POST", {
      environmentId: this.environmentId,
      userId
    });
  }
  async update(userId, personInput) {
    return makeRequest(
      this.apiHost,
      `/api/v1/client/${this.environmentId}/people/${userId}`,
      "POST",
      personInput
    );
  }
}
class ResponseAPI {
  constructor(apiHost, environmentId) {
    this.apiHost = apiHost;
    this.environmentId = environmentId;
  }
  async create(responseInput) {
    return makeRequest(this.apiHost, `/api/v1/client/${this.environmentId}/responses`, "POST", responseInput);
  }
  async update({
    responseId,
    finished,
    data,
    ttc
  }) {
    return makeRequest(this.apiHost, `/api/v1/client/${this.environmentId}/responses/${responseId}`, "PUT", {
      finished,
      data,
      ttc
    });
  }
}
class StorageAPI {
  constructor(apiHost, environmentId) {
    this.apiHost = apiHost;
    this.environmentId = environmentId;
  }
  async uploadFile(file, { allowedFileExtensions, surveyId } = {}) {
    if (!(file instanceof Blob) || !(file instanceof File)) {
      throw new Error(`Invalid file type. Expected Blob or File, but received ${typeof file}`);
    }
    const payload = {
      fileName: file.name,
      fileType: file.type,
      allowedFileExtensions,
      surveyId
    };
    const response = await fetch(`${this.apiHost}/api/v1/client/${this.environmentId}/storage`, {
      method: "POST",
      headers: {
        "Content-Type": "application/json"
      },
      body: JSON.stringify(payload)
    });
    if (!response.ok) {
      throw new Error(`Upload failed with status: ${response.status}`);
    }
    const json = await response.json();
    const { data } = json;
    const { signedUrl, fileUrl, signingData, presignedFields, updatedFileName } = data;
    let requestHeaders = {};
    if (signingData) {
      const { signature, timestamp, uuid } = signingData;
      requestHeaders = {
        "X-File-Type": file.type,
        "X-File-Name": encodeURIComponent(updatedFileName),
        "X-Survey-ID": surveyId ?? "",
        "X-Signature": signature,
        "X-Timestamp": String(timestamp),
        "X-UUID": uuid
      };
    }
    const formData = new FormData();
    if (presignedFields) {
      Object.keys(presignedFields).forEach((key) => {
        formData.append(key, presignedFields[key]);
      });
    }
    formData.append("file", file);
    const uploadResponse = await fetch(signedUrl, {
      method: "POST",
      ...signingData ? { headers: requestHeaders } : {},
      body: formData
    });
    if (!uploadResponse.ok) {
      if (signingData) {
        const uploadJson = await uploadResponse.json();
        const error = new Error(uploadJson.message);
        error.name = "FileTooLargeError";
        throw error;
      }
      const errorText = await uploadResponse.text();
      if (presignedFields && errorText && errorText.includes("EntityTooLarge")) {
        const error = new Error("File size exceeds the size limit for your plan");
        error.name = "FileTooLargeError";
        throw error;
      }
      throw new Error(`Upload failed with status: ${uploadResponse.status}`);
    }
    return fileUrl;
  }
}
class Client {
  constructor(options) {
    const { apiHost, environmentId } = options;
    this.response = new ResponseAPI(apiHost, environmentId);
    this.display = new DisplayAPI(apiHost, environmentId);
    this.action = new ActionAPI(apiHost, environmentId);
    this.people = new PeopleAPI(apiHost, environmentId);
    this.storage = new StorageAPI(apiHost, environmentId);
  }
}
class FormbricksAPI {
  constructor(options) {
    this.client = new Client(options);
  }
}
class Logger {
  constructor() {
    this.logLevel = "error";
  }
  static getInstance() {
    if (!Logger.instance) {
      Logger.instance = new Logger();
    }
    return Logger.instance;
  }
  configure(config2) {
    if (config2 && config2.logLevel !== void 0) {
      this.logLevel = config2.logLevel;
    }
  }
  logger(message, level) {
    if (level === "debug" && this.logLevel !== "debug") {
      return;
    }
    const timestamp = (/* @__PURE__ */ new Date()).toISOString();
    const logMessage = `🧱 Formbricks - ${timestamp} [${level.toUpperCase()}] - ${message}`;
    if (level === "error") {
      console.error(logMessage);
    } else {
      console.log(logMessage);
    }
  }
  debug(message) {
    this.logger(message, "debug");
  }
  error(message) {
    this.logger(message, "error");
  }
}
const ok = (value) => ({ ok: true, value });
const okVoid = () => ({ ok: true, value: void 0 });
const err = (error) => ({
  ok: false,
  error
});
function match(result, onSuccess, onError) {
  if (result.ok === true) {
    return onSuccess(result.value);
  }
  return onError(result.error);
}
const wrapThrows = (fn) => (...args) => {
  try {
    return {
      ok: true,
      value: fn(...args)
    };
  } catch (error) {
    return {
      ok: false,
      error
    };
  }
};
const logger$7 = Logger.getInstance();
const _ErrorHandler = class _ErrorHandler {
  constructor(errorHandler2) {
    this.customized = false;
    if (errorHandler2) {
      this.handleError = errorHandler2;
      this.customized = true;
    } else {
      this.handleError = (err2) => Logger.getInstance().error(JSON.stringify(err2));
    }
  }
  static getInstance() {
    if (!_ErrorHandler.instance) {
      _ErrorHandler.instance = new _ErrorHandler();
    }
    return _ErrorHandler.instance;
  }
  static init(errorHandler2) {
    this.initialized = true;
    _ErrorHandler.instance = new _ErrorHandler(errorHandler2);
  }
  printStatus() {
    logger$7.debug(`Custom error handler: ${this.customized ? "yes" : "no"}`);
  }
  handle(error) {
    console.warn("🧱 Formbricks - Global error: ", error);
    this.handleError(error);
  }
};
_ErrorHandler.initialized = false;
let ErrorHandler = _ErrorHandler;
const LOCAL_STORAGE_KEY = "formbricks-js";
class Config {
  constructor() {
    this.config = null;
    const localConfig = this.loadFromLocalStorage();
    if (localConfig.ok) {
      this.config = localConfig.value;
    }
  }
  static getInstance() {
    if (!Config.instance) {
      Config.instance = new Config();
    }
    return Config.instance;
  }
  update(newConfig) {
    if (newConfig) {
      const expiresAt = new Date((/* @__PURE__ */ new Date()).getTime() + 2 * 6e4);
      this.config = {
        ...this.config,
        ...newConfig,
        status: newConfig.status || "success",
        expiresAt
      };
      this.saveToLocalStorage();
    }
  }
  get() {
    if (!this.config) {
      throw new Error("config is null, maybe the init function was not called?");
    }
    return this.config;
  }
  loadFromLocalStorage() {
    if (typeof window !== "undefined") {
      const savedConfig = localStorage.getItem(LOCAL_STORAGE_KEY);
      if (savedConfig) {
        const parsedConfig = JSON.parse(savedConfig);
        if (parsedConfig.expiresAt && new Date(parsedConfig.expiresAt) <= /* @__PURE__ */ new Date()) {
          return err(new Error("Config in local storage has expired"));
        }
        return ok(JSON.parse(savedConfig));
      }
    }
    return err(new Error("No or invalid config in local storage"));
  }
  saveToLocalStorage() {
    return wrapThrows(() => localStorage.setItem(LOCAL_STORAGE_KEY, JSON.stringify(this.config)))();
  }
  // reset the config
  resetConfig() {
    this.config = null;
    return wrapThrows(() => localStorage.removeItem(LOCAL_STORAGE_KEY))();
  }
}
const diffInDays = (date1, date2) => {
  const diffTime = Math.abs(date2.getTime() - date1.getTime());
  return Math.floor(diffTime / (1e3 * 60 * 60 * 24));
};
const getIsDebug = () => window.location.search.includes("formbricksDebug=true");
const config$5 = Config.getInstance();
const logger$6 = Logger.getInstance();
let syncIntervalId = null;
const syncWithBackend = async ({ apiHost, environmentId, userId }, noCache) => {
  try {
    const baseUrl = `${apiHost}/api/v1/client/${environmentId}/in-app/sync`;
    const urlSuffix = `?version=${"1.6.2"}`;
    let fetchOptions = {};
    if (noCache || getIsDebug()) {
      fetchOptions.cache = "no-cache";
      logger$6.debug("No cache option set for sync");
    }
    if (!userId) {
      const url2 = baseUrl + urlSuffix;
      const response2 = await fetch(url2, fetchOptions);
      if (!response2.ok) {
        const jsonRes = await response2.json();
        return err({
          code: "network_error",
          status: response2.status,
          message: "Error syncing with backend",
          url: url2,
          responseMessage: jsonRes.message
        });
      }
      return ok((await response2.json()).data);
    }
    const url = `${baseUrl}/${userId}${urlSuffix}`;
    const response = await fetch(url, fetchOptions);
    if (!response.ok) {
      const jsonRes = await response.json();
      return err({
        code: "network_error",
        status: response.status,
        message: "Error syncing with backend",
        url,
        responseMessage: jsonRes.message
      });
    }
    const data = await response.json();
    const { data: state } = data;
    return ok(state);
  } catch (e) {
    return err(e);
  }
};
const sync = async (params, noCache = false) => {
  var _a;
  try {
    const syncResult = await syncWithBackend(params, noCache);
    if ((syncResult == null ? void 0 : syncResult.ok) !== true) {
      throw syncResult.error;
    }
    let oldState;
    try {
      oldState = config$5.get().state;
    } catch (e) {
    }
    let state = {
      surveys: syncResult.value.surveys,
      noCodeActionClasses: syncResult.value.noCodeActionClasses,
      product: syncResult.value.product,
      attributes: ((_a = syncResult.value.person) == null ? void 0 : _a.attributes) || {}
    };
    if (!params.userId) {
      state = {
        ...state,
        displays: (oldState == null ? void 0 : oldState.displays) || []
      };
      state = filterPublicSurveys(state);
      const surveyNames = state.surveys.map((s) => s.name);
      logger$6.debug("Fetched " + surveyNames.length + " surveys during sync: " + surveyNames.join(", "));
    } else {
      const surveyNames = state.surveys.map((s) => s.name);
      logger$6.debug("Fetched " + surveyNames.length + " surveys during sync: " + surveyNames.join(", "));
    }
    config$5.update({
      apiHost: params.apiHost,
      environmentId: params.environmentId,
      userId: params.userId,
      state
    });
  } catch (error) {
    logger$6.error(`Error during sync: ${error}`);
    throw error;
  }
};
const filterPublicSurveys = (state) => {
  const { displays, product } = state;
  let { surveys } = state;
  if (!displays) {
    return state;
  }
  let filteredSurveys = surveys.filter((survey) => {
    if (survey.displayOption === "respondMultiple") {
      return true;
    } else if (survey.displayOption === "displayOnce") {
      return displays.filter((display) => display.surveyId === survey.id).length === 0;
    } else if (survey.displayOption === "displayMultiple") {
      return displays.filter((display) => display.surveyId === survey.id && display.responded).length === 0;
    } else {
      throw Error("Invalid displayOption");
    }
  });
  const latestDisplay = displays.length > 0 ? displays[displays.length - 1] : void 0;
  filteredSurveys = filteredSurveys.filter((survey) => {
    if (!latestDisplay) {
      return true;
    } else if (survey.recontactDays !== null) {
      const lastDisplaySurvey = displays.filter((display) => display.surveyId === survey.id)[0];
      if (!lastDisplaySurvey) {
        return true;
      }
      return diffInDays(/* @__PURE__ */ new Date(), new Date(lastDisplaySurvey.createdAt)) >= survey.recontactDays;
    } else if (product.recontactDays !== null) {
      return diffInDays(/* @__PURE__ */ new Date(), new Date(latestDisplay.createdAt)) >= product.recontactDays;
    } else {
      return true;
    }
  });
  return {
    ...state,
    surveys: filteredSurveys
  };
};
const addExpiryCheckListener = () => {
  const updateInterval = 1e3 * 30;
  if (typeof window !== "undefined" && syncIntervalId === null) {
    syncIntervalId = window.setInterval(async () => {
      try {
        if (config$5.get().expiresAt && new Date(config$5.get().expiresAt) >= /* @__PURE__ */ new Date()) {
          return;
        }
        logger$6.debug("Config has expired. Starting sync.");
        await sync({
          apiHost: config$5.get().apiHost,
          environmentId: config$5.get().environmentId,
          userId: config$5.get().userId
        });
      } catch (e) {
        logger$6.error(`Error during expiry check: ${e}`);
        logger$6.debug("Extending config and try again later.");
        const existingConfig = config$5.get();
        config$5.update(existingConfig);
      }
    }, updateInterval);
  }
};
const removeExpiryCheckListener = () => {
  if (typeof window !== "undefined" && syncIntervalId !== null) {
    window.clearInterval(syncIntervalId);
    syncIntervalId = null;
  }
};
const delay = (ms) => {
  return new Promise((resolve) => setTimeout(resolve, ms));
};
class ResponseQueue {
  constructor(config2, surveyState) {
    __publicField(this, "queue", []);
    __publicField(this, "config");
    __publicField(this, "surveyState");
    __publicField(this, "isRequestInProgress", false);
    __publicField(this, "api");
    this.config = config2;
    this.surveyState = surveyState;
    this.api = new FormbricksAPI({
      apiHost: config2.apiHost,
      environmentId: config2.environmentId
    });
  }
  add(responseUpdate) {
    this.surveyState.accumulateResponse(responseUpdate);
    if (this.config.setSurveyState) {
      this.config.setSurveyState(this.surveyState);
    }
    this.queue.push(responseUpdate);
    this.processQueue();
  }
  async processQueue() {
    if (this.isRequestInProgress)
      return;
    if (this.queue.length === 0)
      return;
    this.isRequestInProgress = true;
    const responseUpdate = this.queue[0];
    let attempts = 0;
    while (attempts < this.config.retryAttempts) {
      const success = await this.sendResponse(responseUpdate);
      if (success) {
        this.queue.shift();
        break;
      }
      console.error("Formbricks: Failed to send response. Retrying...", attempts);
      await delay(1e3);
      attempts++;
    }
    if (attempts >= this.config.retryAttempts) {
      console.error("Failed to send response after 2 attempts.");
      if (this.config.onResponseSendingFailed) {
        this.config.onResponseSendingFailed(responseUpdate);
      }
      this.isRequestInProgress = false;
    } else {
      if (responseUpdate.finished && this.config.onResponseSendingFinished) {
        this.config.onResponseSendingFinished();
      }
      this.isRequestInProgress = false;
      this.processQueue();
    }
  }
  async sendResponse(responseUpdate) {
    try {
      if (this.surveyState.responseId !== null) {
        await this.api.client.response.update({ ...responseUpdate, responseId: this.surveyState.responseId });
      } else {
        const response = await this.api.client.response.create({
          ...responseUpdate,
          surveyId: this.surveyState.surveyId,
          userId: this.surveyState.userId || null,
          singleUseId: this.surveyState.singleUseId || null
        });
        if (!response.ok) {
          throw new Error("Could not create response");
        }
        if (this.surveyState.displayId) {
          try {
            await this.api.client.display.update(this.surveyState.displayId, {
              responseId: response.data.id
            });
          } catch (error) {
            console.error("Failed to update display, proceeding with the response.", error);
          }
        }
        this.surveyState.updateResponseId(response.data.id);
        if (this.config.setSurveyState) {
          this.config.setSurveyState(this.surveyState);
        }
      }
      return true;
    } catch (error) {
      console.error(error);
      return false;
    }
  }
  // update surveyState
  updateSurveyState(surveyState) {
    this.surveyState = surveyState;
  }
}
class SurveyState {
  constructor(surveyId, singleUseId, responseId, userId) {
    __publicField(this, "responseId", null);
    __publicField(this, "displayId", null);
    __publicField(this, "userId", null);
    __publicField(this, "surveyId");
    __publicField(this, "responseAcc", { finished: false, data: {}, ttc: {} });
    __publicField(this, "singleUseId");
    this.surveyId = surveyId;
    this.userId = userId ?? null;
    this.singleUseId = singleUseId ?? null;
    this.responseId = responseId ?? null;
  }
  /**
   * Set the current survey ID
   * @param id - The survey ID
   */
  setSurveyId(id) {
    this.surveyId = id;
    this.clear();
  }
  /**
   * Get a copy of the current state
   */
  copy() {
    const copyInstance = new SurveyState(
      this.surveyId,
      this.singleUseId ?? void 0,
      this.responseId ?? void 0,
      this.userId ?? void 0
    );
    copyInstance.responseId = this.responseId;
    copyInstance.responseAcc = this.responseAcc;
    return copyInstance;
  }
  /**
   * Update the response ID after a successful response creation
   * @param id - The response ID
   */
  updateResponseId(id) {
    this.responseId = id;
  }
  /**
   * Update the response ID after a successful response creation
   * @param id - The response ID
   */
  updateDisplayId(id) {
    this.displayId = id;
  }
  /**
   * Update the user ID
   * @param id - The user ID
   */
  updateUserId(id) {
    this.userId = id;
  }
  /**
   * Accumulate the responses
   * @param responseUpdate - The new response data to add
   */
  accumulateResponse(responseUpdate) {
    this.responseAcc = {
      finished: responseUpdate.finished,
      ttc: responseUpdate.ttc,
      data: { ...this.responseAcc.data, ...responseUpdate.data }
    };
  }
  /**
   * Check if the current accumulated response is finished
   */
  isResponseFinished() {
    return this.responseAcc.finished;
  }
  /**
   * Clear the current state
   */
  clear() {
    this.responseId = null;
    this.responseAcc = { finished: false, data: {}, ttc: {} };
  }
}
const SurveyState$1 = SurveyState;
let exitIntentListenerAdded = false;
let exitIntentListenerWrapper = async function(e) {
  if (e.clientY <= 0) {
    const trackResult = await trackAction("Exit Intent (Desktop)");
    if (trackResult.ok !== true) {
      return err(trackResult.error);
    }
  }
};
const addExitIntentListener = () => {
  if (typeof document !== "undefined" && !exitIntentListenerAdded) {
    document.querySelector("body").addEventListener("mouseleave", exitIntentListenerWrapper);
    exitIntentListenerAdded = true;
  }
};
const removeExitIntentListener = () => {
  if (exitIntentListenerAdded) {
    document.removeEventListener("mouseleave", exitIntentListenerWrapper);
    exitIntentListenerAdded = false;
  }
};
let scrollDepthListenerAdded = false;
let scrollDepthTriggered = false;
let scrollDepthListenerWrapper = async () => {
  const scrollPosition = window.scrollY;
  const windowSize = window.innerHeight;
  const bodyHeight = document.documentElement.scrollHeight;
  if (scrollPosition === 0) {
    scrollDepthTriggered = false;
  }
  if (!scrollDepthTriggered && scrollPosition / (bodyHeight - windowSize) >= 0.5) {
    scrollDepthTriggered = true;
    const trackResult = await trackAction("50% Scroll");
    if (trackResult.ok !== true) {
      return err(trackResult.error);
    }
  }
};
const addScrollDepthListener = () => {
  if (typeof window !== "undefined" && !scrollDepthListenerAdded) {
    window.addEventListener("load", () => {
      window.addEventListener("scroll", scrollDepthListenerWrapper);
    });
    scrollDepthListenerAdded = true;
  }
};
const removeScrollDepthListener = () => {
  if (scrollDepthListenerAdded) {
    window.removeEventListener("scroll", scrollDepthListenerWrapper);
    scrollDepthListenerAdded = false;
  }
};
const config$4 = Config.getInstance();
const logger$5 = Logger.getInstance();
const errorHandler$1 = ErrorHandler.getInstance();
const checkPageUrl = async () => {
  var _a;
  logger$5.debug(`Checking page url: ${window.location.href}`);
  const { state } = config$4.get();
  const { noCodeActionClasses = [], surveys = [] } = state ?? {};
  const actionsWithPageUrl = noCodeActionClasses.filter((action) => {
    const { innerHtml, cssSelector, pageUrl } = action.noCodeConfig || {};
    return pageUrl && !innerHtml && !cssSelector;
  });
  const surveysWithInlineTriggers = surveys.filter((survey) => {
    var _a2;
    const { pageUrl, cssSelector, innerHtml } = ((_a2 = survey.inlineTriggers) == null ? void 0 : _a2.noCodeConfig) || {};
    return pageUrl && !cssSelector && !innerHtml;
  });
  if (actionsWithPageUrl.length > 0) {
    for (const event of actionsWithPageUrl) {
      if (!((_a = event.noCodeConfig) == null ? void 0 : _a.pageUrl)) {
        continue;
      }
      const {
        noCodeConfig: { pageUrl }
      } = event;
      const match2 = checkUrlMatch(window.location.href, pageUrl.value, pageUrl.rule);
      if (match2.ok !== true)
        return err(match2.error);
      if (match2.value === false)
        continue;
      const trackResult = await trackAction(event.name);
      if (trackResult.ok !== true)
        return err(trackResult.error);
    }
  }
  if (surveysWithInlineTriggers.length > 0) {
    surveysWithInlineTriggers.forEach((survey) => {
      const { noCodeConfig } = survey.inlineTriggers ?? {};
      const { pageUrl } = noCodeConfig ?? {};
      if (pageUrl) {
        const match2 = checkUrlMatch(window.location.href, pageUrl.value, pageUrl.rule);
        if (match2.ok !== true)
          return err(match2.error);
        renderWidget(survey);
      }
    });
  }
  return okVoid();
};
let arePageUrlEventListenersAdded = false;
const checkPageUrlWrapper = () => checkPageUrl();
const addPageUrlEventListeners = () => {
  if (typeof window === "undefined" || arePageUrlEventListenersAdded)
    return;
  window.addEventListener("hashchange", checkPageUrlWrapper);
  window.addEventListener("popstate", checkPageUrlWrapper);
  window.addEventListener("pushstate", checkPageUrlWrapper);
  window.addEventListener("replacestate", checkPageUrlWrapper);
  window.addEventListener("load", checkPageUrlWrapper);
  arePageUrlEventListenersAdded = true;
};
const removePageUrlEventListeners = () => {
  if (typeof window === "undefined" || !arePageUrlEventListenersAdded)
    return;
  window.removeEventListener("hashchange", checkPageUrlWrapper);
  window.removeEventListener("popstate", checkPageUrlWrapper);
  window.removeEventListener("pushstate", checkPageUrlWrapper);
  window.removeEventListener("replacestate", checkPageUrlWrapper);
  window.removeEventListener("load", checkPageUrlWrapper);
  arePageUrlEventListenersAdded = false;
};
function checkUrlMatch(url, pageUrlValue, pageUrlRule) {
  switch (pageUrlRule) {
    case "exactMatch":
      return ok(url === pageUrlValue);
    case "contains":
      return ok(url.includes(pageUrlValue));
    case "startsWith":
      return ok(url.startsWith(pageUrlValue));
    case "endsWith":
      return ok(url.endsWith(pageUrlValue));
    case "notMatch":
      return ok(url !== pageUrlValue);
    case "notContains":
      return ok(!url.includes(pageUrlValue));
    default:
      return err({
        code: "invalid_match_type",
        message: "Invalid match type"
      });
  }
}
const evaluateNoCodeConfig = (targetElement, action) => {
  var _a, _b, _c, _d, _e, _f, _g, _h;
  const innerHtml = (_b = (_a = action.noCodeConfig) == null ? void 0 : _a.innerHtml) == null ? void 0 : _b.value;
  const cssSelectors = (_d = (_c = action.noCodeConfig) == null ? void 0 : _c.cssSelector) == null ? void 0 : _d.value;
  const pageUrl = (_f = (_e = action.noCodeConfig) == null ? void 0 : _e.pageUrl) == null ? void 0 : _f.value;
  const pageUrlRule = (_h = (_g = action.noCodeConfig) == null ? void 0 : _g.pageUrl) == null ? void 0 : _h.rule;
  if (!innerHtml && !cssSelectors && !pageUrl) {
    return false;
  }
  if (innerHtml && targetElement.innerHTML !== innerHtml) {
    return false;
  }
  if (cssSelectors) {
    const individualSelectors = cssSelectors.split(/\s*(?=[.#])/);
    for (let selector of individualSelectors) {
      if (!targetElement.matches(selector)) {
        return false;
      }
    }
  }
  if (pageUrl && pageUrlRule) {
    const urlMatch = checkUrlMatch(window.location.href, pageUrl, pageUrlRule);
    if (!urlMatch.ok || !urlMatch.value) {
      return false;
    }
  }
  return true;
};
const checkClickMatch = (event) => {
  const { state } = config$4.get();
  if (!state) {
    return;
  }
  const { noCodeActionClasses } = state;
  if (!noCodeActionClasses) {
    return;
  }
  const targetElement = event.target;
  noCodeActionClasses.forEach((action) => {
    const shouldTrack = evaluateNoCodeConfig(targetElement, action);
    if (shouldTrack) {
      trackAction(action.name).then((res) => {
        match(
          res,
          (_value) => {
          },
          (err2) => {
            errorHandler$1.handle(err2);
          }
        );
      });
    }
  });
  const activeSurveys = state.surveys;
  if (!activeSurveys || activeSurveys.length === 0) {
    return;
  }
  activeSurveys.forEach((survey) => {
    const { inlineTriggers } = survey;
    if (inlineTriggers) {
      const shouldTrack = evaluateNoCodeConfig(targetElement, inlineTriggers);
      if (shouldTrack) {
        renderWidget(survey);
      }
    }
  });
};
let isClickEventListenerAdded = false;
const checkClickMatchWrapper = (e) => checkClickMatch(e);
const addClickEventListener = () => {
  if (typeof window === "undefined" || isClickEventListenerAdded)
    return;
  document.addEventListener("click", checkClickMatchWrapper);
  isClickEventListenerAdded = true;
};
const removeClickEventListener = () => {
  if (!isClickEventListenerAdded)
    return;
  document.removeEventListener("click", checkClickMatchWrapper);
  isClickEventListenerAdded = false;
};
let areRemoveEventListenersAdded = false;
const addEventListeners = () => {
  addExpiryCheckListener();
  addPageUrlEventListeners();
  addClickEventListener();
  addExitIntentListener();
  addScrollDepthListener();
};
const addCleanupEventListeners = () => {
  if (areRemoveEventListenersAdded)
    return;
  window.addEventListener("beforeunload", () => {
    removeExpiryCheckListener();
    removePageUrlEventListeners();
    removeClickEventListener();
    removeExitIntentListener();
    removeScrollDepthListener();
  });
  areRemoveEventListenersAdded = true;
};
const removeCleanupEventListeners = () => {
  if (!areRemoveEventListenersAdded)
    return;
  window.removeEventListener("beforeunload", () => {
    removeExpiryCheckListener();
    removePageUrlEventListeners();
    removeClickEventListener();
    removeExitIntentListener();
    removeScrollDepthListener();
  });
  areRemoveEventListenersAdded = false;
};
const removeAllEventListeners = () => {
  removeExpiryCheckListener();
  removePageUrlEventListeners();
  removeClickEventListener();
  removeExitIntentListener();
  removeScrollDepthListener();
  removeCleanupEventListeners();
};
const config$3 = Config.getInstance();
const logger$4 = Logger.getInstance();
const updatePersonAttribute = async (key, value) => {
  const { apiHost, environmentId, userId } = config$3.get();
  if (!userId) {
    return err({
      code: "missing_person",
      message: "Unable to update attribute. User identification deactivated. No userId set."
    });
  }
  const input = {
    attributes: {
      [key]: value
    }
  };
  const api = new FormbricksAPI({
    apiHost,
    environmentId
  });
  const res = await api.client.people.update(userId, input);
  if (!res.ok) {
    return err({
      code: "network_error",
      status: 500,
      message: `Error updating person with userId ${userId}`,
      url: `${config$3.get().apiHost}/api/v1/client/${environmentId}/people/${userId}`,
      responseMessage: res.error.message
    });
  }
  if (res.data.changed) {
    logger$4.debug("Attribute updated in Formbricks");
  }
  return okVoid();
};
const updatePersonAttributes = async (apiHost, environmentId, userId, attributes) => {
  var _a, _b;
  if (!userId) {
    return err({
      code: "missing_person",
      message: "Unable to update attribute. User identification deactivated. No userId set."
    });
  }
  const updatedAttributes = { ...attributes };
  try {
    const existingAttributes = (_b = (_a = config$3.get()) == null ? void 0 : _a.state) == null ? void 0 : _b.attributes;
    if (existingAttributes) {
      for (const [key, value] of Object.entries(existingAttributes)) {
        if (updatedAttributes[key] === value) {
          delete updatedAttributes[key];
        }
      }
    }
  } catch (e) {
    logger$4.debug("config not set; sending all attributes to backend");
  }
  if (Object.keys(updatedAttributes).length === 0) {
    logger$4.debug("No attributes to update. Skipping update.");
    return ok(updatedAttributes);
  }
  logger$4.debug("Updating attributes: " + JSON.stringify(updatedAttributes));
  const input = {
    attributes: updatedAttributes
  };
  const api = new FormbricksAPI({
    apiHost,
    environmentId
  });
  const res = await api.client.people.update(userId, input);
  if (res.ok) {
    return ok(updatedAttributes);
  }
  return err({
    code: "network_error",
    status: 500,
    message: `Error updating person with userId ${userId}`,
    url: `${apiHost}/api/v1/client/${environmentId}/people/${userId}`,
    responseMessage: res.error.message
  });
};
const isExistingAttribute = (key, value) => {
  if (config$3.get().state.attributes[key] === value) {
    return true;
  }
  return false;
};
const setPersonUserId = async () => {
  logger$4.error("'setUserId' is no longer supported. Please set the userId in the init call instead.");
  return okVoid();
};
const setPersonAttribute = async (key, value) => {
  logger$4.debug("Setting attribute: " + key + " to value: " + value);
  if (isExistingAttribute(key, value.toString())) {
    logger$4.debug("Attribute already set to this value. Skipping update.");
    return okVoid();
  }
  const result = await updatePersonAttribute(key, value.toString());
  if (result.ok) {
    config$3.update({
      environmentId: config$3.get().environmentId,
      apiHost: config$3.get().apiHost,
      userId: config$3.get().userId,
      state: {
        ...config$3.get().state,
        attributes: {
          ...config$3.get().state.attributes,
          [key]: value.toString()
        }
      }
    });
    return okVoid();
  }
  return err(result.error);
};
const logoutPerson = async () => {
  deinitalize();
  config$3.resetConfig();
};
const resetPerson = async () => {
  logger$4.debug("Resetting state & getting new state from backend");
  closeSurvey();
  const syncParams = {
    environmentId: config$3.get().environmentId,
    apiHost: config$3.get().apiHost,
    userId: config$3.get().userId
  };
  await logoutPerson();
  try {
    await initialize(syncParams);
    return okVoid();
  } catch (e) {
    return err(e);
  }
};
const config$2 = Config.getInstance();
const logger$3 = Logger.getInstance();
let isInitialized = false;
const setIsInitialized = (value) => {
  isInitialized = value;
};
const initialize = async (c) => {
  if (getIsDebug()) {
    logger$3.configure({ logLevel: "debug" });
  }
  if (isInitialized) {
    logger$3.debug("Already initialized, skipping initialization.");
    return okVoid();
  }
  let existingConfig;
  try {
    existingConfig = config$2.get();
    logger$3.debug("Found existing configuration.");
  } catch (e) {
    logger$3.debug("No existing configuration found.");
  }
  if ((existingConfig == null ? void 0 : existingConfig.status) === "error") {
    logger$3.debug("Formbricks was set to an error state.");
    if ((existingConfig == null ? void 0 : existingConfig.expiresAt) && new Date(existingConfig.expiresAt) > /* @__PURE__ */ new Date()) {
      logger$3.debug("Error state is not expired, skipping initialization");
      return okVoid();
    } else {
      logger$3.debug("Error state is expired. Continue with initialization.");
    }
  }
  ErrorHandler.getInstance().printStatus();
  logger$3.debug("Start initialize");
  if (!c.environmentId) {
    logger$3.debug("No environmentId provided");
    return err({
      code: "missing_field",
      field: "environmentId"
    });
  }
  if (!c.apiHost) {
    logger$3.debug("No apiHost provided");
    return err({
      code: "missing_field",
      field: "apiHost"
    });
  }
  logger$3.debug("Adding widget container to DOM");
  addWidgetContainer();
  if (!c.userId && c.attributes) {
    logger$3.error("No userId provided but attributes. Cannot update attributes without userId.");
    return err({
      code: "missing_field",
      field: "userId"
    });
  }
  let updatedAttributes = null;
  if (c.userId && c.attributes) {
    const res = await updatePersonAttributes(c.apiHost, c.environmentId, c.userId, c.attributes);
    if (res.ok !== true) {
      return err(res.error);
    }
    updatedAttributes = res.value;
  }
  if (existingConfig && existingConfig.state && existingConfig.environmentId === c.environmentId && existingConfig.apiHost === c.apiHost && existingConfig.userId === c.userId && existingConfig.expiresAt) {
    logger$3.debug("Configuration fits init parameters.");
    if (existingConfig.expiresAt < /* @__PURE__ */ new Date()) {
      logger$3.debug("Configuration expired.");
      try {
        await sync({
          apiHost: c.apiHost,
          environmentId: c.environmentId,
          userId: c.userId
        });
      } catch (e) {
        putFormbricksInErrorState();
      }
    } else {
      logger$3.debug("Configuration not expired. Extending expiration.");
      config$2.update(existingConfig);
    }
  } else {
    logger$3.debug(
      "No valid configuration found or it has been expired. Resetting config and creating new one."
    );
    config$2.resetConfig();
    logger$3.debug("Syncing.");
    try {
      await sync({
        apiHost: c.apiHost,
        environmentId: c.environmentId,
        userId: c.userId
      });
    } catch (e) {
      handleErrorOnFirstInit();
    }
    await trackAction("New Session");
  }
  if (updatedAttributes && Object.keys(updatedAttributes).length > 0) {
    config$2.update({
      environmentId: config$2.get().environmentId,
      apiHost: config$2.get().apiHost,
      userId: config$2.get().userId,
      state: {
        ...config$2.get().state,
        attributes: { ...config$2.get().state.attributes, ...c.attributes }
      }
    });
  }
  logger$3.debug("Adding event listeners");
  addEventListeners();
  addCleanupEventListeners();
  setIsInitialized(true);
  logger$3.debug("Initialized");
  checkPageUrl();
  return okVoid();
};
const handleErrorOnFirstInit = () => {
  const initialErrorConfig = {
    status: "error",
    expiresAt: new Date((/* @__PURE__ */ new Date()).getTime() + 10 * 6e4)
    // 10 minutes in the future
  };
  wrapThrows(() => localStorage.setItem(LOCAL_STORAGE_KEY, JSON.stringify(initialErrorConfig)))();
  throw new Error("Could not initialize formbricks");
};
const checkInitialized = () => {
  logger$3.debug("Check if initialized");
  if (!isInitialized || !ErrorHandler.initialized) {
    return err({
      code: "not_initialized",
      message: "Formbricks not initialized. Call initialize() first."
    });
  }
  return okVoid();
};
const deinitalize = () => {
  logger$3.debug("Deinitializing");
  removeWidgetContainer();
  setIsSurveyRunning(false);
  removeAllEventListeners();
  setIsInitialized(false);
};
const putFormbricksInErrorState = () => {
  logger$3.debug("Putting formbricks in error state");
  config$2.update({
    ...config$2.get(),
    status: "error",
    expiresAt: new Date((/* @__PURE__ */ new Date()).getTime() + 10 * 6e4)
    // 10 minutes in the future
  });
  deinitalize();
};
const containerId = "formbricks-web-container";
const config$1 = Config.getInstance();
const logger$2 = Logger.getInstance();
const errorHandler = ErrorHandler.getInstance();
let isSurveyRunning = false;
let setIsError = (_) => {
};
let setIsResponseSendingFinished = (_) => {
};
const setIsSurveyRunning = (value) => {
  isSurveyRunning = value;
};
const renderWidget = async (survey) => {
  if (isSurveyRunning) {
    logger$2.debug("A survey is already running. Skipping.");
    return;
  }
  setIsSurveyRunning(true);
  if (survey.delay) {
    logger$2.debug(`Delaying survey by ${survey.delay} seconds.`);
  }
  const product = config$1.get().state.product;
  const surveyState = new SurveyState$1(survey.id, null, null, config$1.get().userId);
  const responseQueue = new ResponseQueue(
    {
      apiHost: config$1.get().apiHost,
      environmentId: config$1.get().environmentId,
      retryAttempts: 2,
      onResponseSendingFailed: () => {
        setIsError(true);
      },
      onResponseSendingFinished: () => {
        setIsResponseSendingFinished(true);
      }
    },
    surveyState
  );
  const productOverwrites = survey.productOverwrites ?? {};
  const brandColor = productOverwrites.brandColor ?? product.brandColor;
  const highlightBorderColor = productOverwrites.highlightBorderColor ?? product.highlightBorderColor;
  const clickOutside = productOverwrites.clickOutsideClose ?? product.clickOutsideClose;
  const darkOverlay = productOverwrites.darkOverlay ?? product.darkOverlay;
  const placement = productOverwrites.placement ?? product.placement;
  const isBrandingEnabled = product.inAppSurveyBranding;
  const formbricksSurveys = await loadFormbricksSurveysExternally();
  setTimeout(() => {
    formbricksSurveys.renderSurveyModal({
      survey,
      brandColor,
      isBrandingEnabled,
      clickOutside,
      darkOverlay,
      highlightBorderColor,
      placement,
      getSetIsError: (f) => {
        setIsError = f;
      },
      getSetIsResponseSendingFinished: (f) => {
        setIsResponseSendingFinished = f;
      },
      onDisplay: async () => {
        const { userId } = config$1.get();
        if (!userId) {
          const localDisplay = {
            createdAt: /* @__PURE__ */ new Date(),
            surveyId: survey.id,
            responded: false
          };
          const existingDisplays = config$1.get().state.displays;
          const displays = existingDisplays ? [...existingDisplays, localDisplay] : [localDisplay];
          const previousConfig = config$1.get();
          let state = filterPublicSurveys({
            ...previousConfig.state,
            displays
          });
          config$1.update({
            ...previousConfig,
            state
          });
        }
        const api = new FormbricksAPI({
          apiHost: config$1.get().apiHost,
          environmentId: config$1.get().environmentId
        });
        const res = await api.client.display.create({
          surveyId: survey.id,
          userId
        });
        if (!res.ok) {
          throw new Error("Could not create display");
        }
        const { id } = res.data;
        surveyState.updateDisplayId(id);
        responseQueue.updateSurveyState(surveyState);
      },
      onResponse: (responseUpdate) => {
        const { userId } = config$1.get();
        if (!userId) {
          const displays = config$1.get().state.displays;
          const lastDisplay = displays && displays[displays.length - 1];
          if (!lastDisplay) {
            throw new Error("No lastDisplay found");
          }
          if (!lastDisplay.responded) {
            lastDisplay.responded = true;
            const previousConfig = config$1.get();
            let state = filterPublicSurveys({
              ...previousConfig.state,
              displays
            });
            config$1.update({
              ...previousConfig,
              state
            });
          }
        }
        if (userId) {
          surveyState.updateUserId(userId);
        }
        responseQueue.updateSurveyState(surveyState);
        responseQueue.add({
          data: responseUpdate.data,
          ttc: responseUpdate.ttc,
          finished: responseUpdate.finished
        });
      },
      onClose: closeSurvey,
      onFileUpload: async (file, params) => {
        const api = new FormbricksAPI({
          apiHost: config$1.get().apiHost,
          environmentId: config$1.get().environmentId
        });
        return await api.client.storage.uploadFile(file, params);
      },
      onRetry: () => {
        setIsError(false);
        responseQueue.processQueue();
      }
    });
  }, survey.delay * 1e3);
};
const closeSurvey = async () => {
  removeWidgetContainer();
  addWidgetContainer();
  if (!config$1.get().userId) {
    const state = config$1.get().state;
    const updatedState = filterPublicSurveys(state);
    config$1.update({
      ...config$1.get(),
      state: updatedState
    });
    setIsSurveyRunning(false);
    return;
  }
  try {
    await sync(
      {
        apiHost: config$1.get().apiHost,
        environmentId: config$1.get().environmentId,
        userId: config$1.get().userId
      },
      true
    );
    setIsSurveyRunning(false);
  } catch (e) {
    errorHandler.handle(e);
    putFormbricksInErrorState();
  }
};
const addWidgetContainer = () => {
  const containerElement = document.createElement("div");
  containerElement.id = containerId;
  document.body.appendChild(containerElement);
};
const removeWidgetContainer = () => {
  var _a;
  (_a = document.getElementById(containerId)) == null ? void 0 : _a.remove();
};
const loadFormbricksSurveysExternally = () => {
  const formbricksSurveysScriptSrc = "https://unpkg.com/@formbricks/surveys@^1.6.1/dist/index.umd.js";
  return new Promise((resolve, reject) => {
    if (window.formbricksSurveys) {
      resolve(window.formbricksSurveys);
    } else {
      const script = document.createElement("script");
      script.src = formbricksSurveysScriptSrc;
      script.async = true;
      script.onload = () => resolve(window.formbricksSurveys);
      script.onerror = (error) => {
        console.error("Failed to load Formbricks Surveys library:", error);
        reject(error);
      };
      document.head.appendChild(script);
    }
  });
};
const logger$1 = Logger.getInstance();
const config = Config.getInstance();
const intentsToNotCreateOnApp = ["Exit Intent (Desktop)", "50% Scroll"];
const shouldDisplayBasedOnPercentage = (displayPercentage) => {
  const randomNum = Math.floor(Math.random() * 100) + 1;
  return randomNum <= displayPercentage;
};
const trackAction = async (name) => {
  var _a;
  const {
    userId,
    state: { surveys = [] }
  } = config.get();
  surveys.forEach(async (survey) => {
    const { inlineTriggers } = survey;
    const { codeConfig } = inlineTriggers ?? {};
    if (name === (codeConfig == null ? void 0 : codeConfig.identifier)) {
      await renderWidget(survey);
      return;
    }
  });
  const input = {
    environmentId: config.get().environmentId,
    userId,
    name
  };
  if (userId && !intentsToNotCreateOnApp.includes(name)) {
    logger$1.debug(`Sending action "${name}" to backend`);
    const api = new FormbricksAPI({
      apiHost: config.get().apiHost,
      environmentId: config.get().environmentId
    });
    const res = await api.client.action.create({
      ...input,
      userId
    });
    if (!res.ok) {
      return err({
        code: "network_error",
        message: `Error tracking action ${name}`,
        status: 500,
        url: `${config.get().apiHost}/api/v1/client/${config.get().environmentId}/actions`,
        responseMessage: res.error.message
      });
    }
    if (getIsDebug()) {
      await sync(
        {
          environmentId: config.get().environmentId,
          apiHost: config.get().apiHost,
          userId
        },
        true
      );
    }
  }
  logger$1.debug(`Formbricks: Action "${name}" tracked`);
  const activeSurveys = (_a = config.get().state) == null ? void 0 : _a.surveys;
  if (!!activeSurveys && activeSurveys.length > 0) {
    await triggerSurvey(name, activeSurveys);
  } else {
    logger$1.debug("No active surveys to display");
  }
  return okVoid();
};
const triggerSurvey = async (actionName, activeSurveys) => {
  for (const survey of activeSurveys) {
    if (survey.displayPercentage) {
      const shouldDisplaySurvey = shouldDisplayBasedOnPercentage(survey.displayPercentage);
      if (!shouldDisplaySurvey) {
        logger$1.debug("Survey display skipped based on displayPercentage.");
        continue;
      }
    }
    for (const trigger of survey.triggers) {
      if (trigger === actionName) {
        logger$1.debug(`Formbricks: survey ${survey.id} triggered by action "${actionName}"`);
        await renderWidget(survey);
        return;
      }
    }
  }
};
const getApi = () => {
  const config2 = Config.getInstance();
  const { environmentId, apiHost } = config2.get();
  if (!environmentId || !apiHost) {
    throw new Error("formbricks.init() must be called before getApi()");
  }
  return new FormbricksAPI({
    apiHost,
    environmentId
  });
};
const wrapThrowsAsync = (fn) => async (...args) => {
  try {
    return {
      ok: true,
      data: await fn(...args)
    };
  } catch (error) {
    return {
      ok: false,
      error
    };
  }
};
class CommandQueue {
  constructor() {
    this.queue = [];
    this.running = false;
    this.resolvePromise = null;
    this.commandPromise = null;
  }
  add(checkInitialized2 = true, command, ...args) {
    this.queue.push({ command, checkInitialized: checkInitialized2, commandArgs: args });
    if (!this.running) {
      this.commandPromise = new Promise((resolve) => {
        this.resolvePromise = resolve;
        this.run();
      });
    }
  }
  async wait() {
    if (this.running) {
      await this.commandPromise;
    }
  }
  async run() {
    this.running = true;
    while (this.queue.length > 0) {
      const errorHandler2 = ErrorHandler.getInstance();
      const currentItem = this.queue.shift();
      if (!currentItem)
        continue;
      if (currentItem.checkInitialized) {
        const initResult = checkInitialized();
        if (initResult && initResult.ok !== true) {
          errorHandler2.handle(initResult.error);
          continue;
        }
      }
      const executeCommand = async () => {
        return await (currentItem == null ? void 0 : currentItem.command.apply(null, currentItem == null ? void 0 : currentItem.commandArgs));
      };
      const result = await wrapThrowsAsync(executeCommand)();
      if (!result)
        continue;
      if (result.ok) {
        if (result.data && !result.data.ok) {
          errorHandler2.handle(result.data.error);
        }
      }
      if (result.ok !== true) {
        errorHandler2.handle(result.error);
      }
    }
    this.running = false;
    if (this.resolvePromise) {
      this.resolvePromise();
      this.resolvePromise = null;
      this.commandPromise = null;
    }
  }
}
const logger = Logger.getInstance();
logger.debug("Create command queue");
const queue = new CommandQueue();
const init = async (initConfig) => {
  ErrorHandler.init(initConfig.errorHandler);
  queue.add(false, initialize, initConfig);
  await queue.wait();
};
const setUserId = async () => {
  queue.add(true, setPersonUserId);
  await queue.wait();
};
const setEmail = async (email) => {
  setAttribute("email", email);
  await queue.wait();
};
const setAttribute = async (key, value) => {
  queue.add(true, setPersonAttribute, key, value);
  await queue.wait();
};
const logout = async () => {
  queue.add(true, logoutPerson);
  await queue.wait();
};
const reset = async () => {
  queue.add(true, resetPerson);
  await queue.wait();
};
const track = async (name, properties = {}) => {
  queue.add(true, trackAction, name, properties);
  await queue.wait();
};
const registerRouteChange = async () => {
  queue.add(true, checkPageUrl);
  await queue.wait();
};
const formbricks = {
  init,
  setUserId,
  setEmail,
  setAttribute,
  track,
  logout,
  reset,
  registerRouteChange,
  getApi
};
export {
  formbricks as default
};
//# sourceMappingURL=index.mjs.map
